import subprocess

def clean_and_format_code(path):
    # Remove unused imports
    autoflake_command = [
        "autoflake",
        "--remove-all-unused-imports",
        "--recursive",
        "--remove-unused-variables",
        "--in-place",
        path,
    ]
    subprocess.run(autoflake_command)

    # Sort imports
    isort_command = ["isort", path]
    subprocess.run(isort_command)
    
    # Reformat files
    black_command = ["black", path]
    subprocess.run(black_command)


import os.path as op
import os
import shutil
from jinja2 import Environment, FileSystemLoader, Undefined
import git

from genesia.clean_and_format_code import clean_and_format_code

available_templates_url = {
    "django": "https://github.com/genesia-dev/django-template.git"
}

available_templates = available_templates_url.keys()


class SilentUndefined(Undefined):
    def __init__(self, name, *args, **kwargs):
        super().__init__(name, *args, **kwargs)
        self.name = name

    def _fail_with_undefined_error(self, *args, **kwargs):
        name = self.name

        class EmptyString(str):
            def __call__(self, *args, **kwargs):
                return f"{{{{name}}}}"

        return EmptyString()

    __add__ = (
        __radd__
    ) = (
        __mul__
    ) = (
        __rmul__
    ) = (
        __div__
    ) = (
        __rdiv__
    ) = (
        __truediv__
    ) = (
        __rtruediv__
    ) = (
        __floordiv__
    ) = (
        __rfloordiv__
    ) = (
        __mod__
    ) = (
        __rmod__
    ) = (
        __pos__
    ) = (
        __neg__
    ) = (
        __call__
    ) = (
        __getitem__
    ) = (
        __lt__
    ) = (
        __le__
    ) = (
        __gt__
    ) = (
        __ge__
    ) = (
        __int__
    ) = __float__ = __complex__ = __pow__ = __rpow__ = _fail_with_undefined_error


def fill_jinja_templates(template_path: str, data: dict, output_path: str) -> None:
    # Iterate through all files and subdirectories
    for root, _, files in os.walk(template_path):
        for file_name in files:
            if file_name.endswith(".jinja"):
                full_path = os.path.join(root, file_name)
                filled_template = fill_jinja_template(full_path, data)

                # Create the corresponding output directory and file
                relative_path = os.path.relpath(root, template_path)
                output_dir = os.path.join(output_path, relative_path)
                os.makedirs(output_dir, exist_ok=True)
                output_file_path = os.path.join(
                    output_dir, file_name.replace(".jinja", "")
                )

                with open(output_file_path, "w") as output_file:
                    output_file.write(filled_template)


def fill_jinja_template(template_path: str, data: dict) -> str:
    # Get the path and filename separately
    base_dir = os.path.dirname(template_path)
    template_name = os.path.basename(template_path)

    # Create a Jinja2 environment and load the template
    env = Environment(loader=FileSystemLoader(base_dir), undefined=SilentUndefined)
    env.filters["pascal_case"] = pascal_case
    
    
    ### TEMPORARY, ALLOW TO CREATE A NEW FIELD AS A PROCESSED VALUE OF ANOTHER FIELD
    import re

    def extract_path_parameters(url_pattern: str):
        matches = re.findall(r'<int:(\w+)>', url_pattern)
        return ', '.join(matches)
    
    env.filters['extract_path_parameters'] = extract_path_parameters
    ### END TEMPORARY
    
    
    env.trim_blocks = True
    env.lstrip_blocks = True
    template = env.get_template(template_name)

    # Fill the template with the provided data
    filled_template = template.render(data)

    return filled_template


def pascal_case(s):
    parts = s.split("_")
    return "".join(part.capitalize() for part in parts)


def create_project_from_template(template_name, project_name, jinja_result):
    assert template_name in available_templates, f"Unknown template {template_name}"
    assert not op.exists(project_name), f"Project {project_name} already exists"

    template_dir = "genesia-templates"

    git.Repo.clone_from(available_templates_url[template_name], template_dir)

    # copy directory
    shutil.copytree(template_dir, project_name)
    # change folder name that are named {{project_name}} to the actual project name
    for root, dirs, files in os.walk(project_name):
        for dir in dirs:
            if "{{project_name}}" in dir:
                shutil.move(
                    op.join(root, dir),
                    op.join(root, dir.replace("{{project_name}}", project_name)),
                )
    jinja_result["project_name"] = project_name
    jinja_result["app_name"] = "app"
    
    # fill jinja templates
    fill_jinja_templates(project_name, jinja_result, project_name)
    # remove jinja templates
    for root, dirs, files in os.walk(project_name):
        for file in files:
            if file.endswith(".jinja"):
                os.remove(op.join(root, file))

    shutil.rmtree(template_dir)
    
    clean_and_format_code(project_name)



import os
from dotenv import load_dotenv
import click
import socketio
import os.path as osp
from yaspin import yaspin
from yaspin.spinners import Spinners

from genesia.templates_manager import create_project_from_template, available_templates

load_dotenv()
ENVIRONMENT = os.getenv("ENVIRONMENT", "production")
OPENAI_API_KEY = os.getenv("OPENAI_API_KEY")

if ENVIRONMENT == "development":
    print("You are using the CLI in development mode. See your .env file.")


class DynamicText:
    def __init__(self, init_value):
        self.value = init_value

    def update(self, new_value):
        self.value = new_value

    def __str__(self):
        return self.value


@click.group()
def cli():
    pass


def validate_openai_key(ctx, param, value):
    if value.startswith("sk"):
        return value
    else:
        raise click.BadParameter("openai-api-key should start with 'sk'")


@cli.command("create")
@click.option("--name", help="Name of the project", required=True)
@click.option(
    "--template",
    type=click.Choice(available_templates),
    help="Template to use (see more https://github.com/ThomasCloarec/genesia-templates)",
    required=True,
)
@click.option(
    "--prompt",
    prompt="Describe what you want to do",
    help="Describe what you want to do",
    required=True,
)
@click.option(
    "--openai-api-key",
    default=OPENAI_API_KEY,
    prompt="OpenAI API key",
    help="OpenAI API key",
    required=True,
    callback=validate_openai_key,
)
def create_project(name, template, prompt, openai_api_key):
    assert not osp.exists(name), f"Project {name} already exists"

    text = DynamicText("Connecting to our AI (it may take a few minutes)...")

    with yaspin(Spinners.pong, text=text, color="yellow") as spinner:
        sio = socketio.Client()

        def on_create_project_response(data):
            if data["status"] == "error":
                print(data["message"])
            elif data["status"] == "success":
                assert "result" in data, "[Internal Error] No result in response"
                result = data["result"]
                create_project_from_template(template, name, result)
                print("Project {} created".format(name))
            sio.disconnect()

        @sio.on("connect")
        def on_connect():
            text.update("Generating your project...")
            sio.emit(
                "create_project",
                {
                    "name": name,
                    "template": template,
                    "prompt": prompt,
                    "api_key": openai_api_key,
                },
                callback=on_create_project_response,
            )

        @sio.on("create_project")
        def on_create_project(msg):
            spinner.write(text)
            text.update(msg)

        @sio.on("disconnect")
        def on_disconnect():
            sio.disconnect()

        if ENVIRONMENT == "development":
            sio.connect("http://127.0.0.1:8000/", wait=False)
        else:
            sio.connect("https://genesia-api.onrender.com/", wait=False)

        try:
            sio.wait()
        except KeyboardInterrupt:
            pass
        finally:
            sio.disconnect()


def run_cli():
    try:
        cli()
    except Exception as e:
        print(f"Error: {e}")


if __name__ == "__main__":
    run_cli()
